

#include <math.h>
#include <stdio.h>
#include "typedefs.h"
#include "dtmf.h"


#define SDMF 0x04
#define MDMF 0x80
#define TIMEHEADER 0x01
#define NUMBERHEADER 0x02
#define NAMEHEADER 0x07

#define MAXCID 48


extern u08 Time10ms;
extern u08 TimeSec;
extern u08 TimeMin;
extern u08 TimeHour;
extern u08 TimeDay;
extern u08 TimeMonth;


u08 cidBytes[MAXCID];

typedef enum {  WAITFORCID, WAITFORU2, WAITFORTYPE,
                WAITFORTYPEONLY, GETLENGTH, INMSG, CHKSUM}cidStates;
extern u08 dspmode;

void CIDbyte(u08 rxbyte, s08 reset)
{
	static u08 cidState = WAITFORCID;
  static u08 cidPointer;
  static u08 cidChecksum;
  static u08 cidLength;

  if (reset == TRUE)
  {
    cidState = WAITFORCID;
    cidBytes[0] = 0;
  }

  switch (cidState)
  {
    case WAITFORCID:
     if (rxbyte == 0x55)
      {
        cidState = WAITFORU2;
      }
      break;

    case WAITFORU2:
      if (rxbyte == 0x55)
      {
        cidState = WAITFORTYPE;
      } else {
        cidState = WAITFORCID;
      }   
      break;

    case WAITFORTYPE:
       if (rxbyte == 0x55)
      {
        break;
      }
      if ( (rxbyte == 0xd5) || (rxbyte == 0xf5) || (rxbyte == 0xfd) )
      {
         cidState = WAITFORTYPEONLY;
         break;
      }

    case WAITFORTYPEONLY:
     if ( (rxbyte == SDMF) || (rxbyte == MDMF) )
      {
        cidChecksum = rxbyte;
        cidBytes[0] = rxbyte;
        cidPointer = 1;
        cidState = GETLENGTH;
      } else {
        cidState = WAITFORCID;
      }   
      break;

    case GETLENGTH:
      if ( rxbyte >= MAXCID )
      {
        cidState = WAITFORCID;
      } else {
        cidBytes[1] = rxbyte;
        cidPointer = 2;
        cidState = INMSG;
        cidChecksum += rxbyte;
        cidLength = rxbyte;
      }
      break;

    case INMSG:
      cidBytes[cidPointer] = rxbyte;
      cidChecksum += rxbyte;
      cidPointer++;
      cidLength--;
      if (cidLength == 0)
      {
        cidState = CHKSUM;
      }  
      break;

    case CHKSUM:
      cidChecksum += rxbyte;
      cidBytes[cidPointer++] = rxbyte;
      cidBytes[cidPointer++] = cidChecksum;
      cidState = WAITFORCID;
      dspmode = FSKEND;
      break;      
  }
}

/* Verify that received CID has a valid checksum */
/* Return TRUE idf the CID frame is valid FALSE otherwise */
s08 checkCIDvalid( void)
{
  u08 i;
  u08 checksum;

  if ( cidBytes[0] != MDMF) return FALSE;
  
  checksum = 0;
  for (i = 0; i == cidBytes[1]; i++) {
    checksum += cidBytes[i+2];
  }

  if (checksum != 0) return FALSE;

  return TRUE;
}

/* Update our time of day clock based on the time from the CID frame
the CID frame must be valid */    
void updateCIDtime()
{
  u08 i;

  i = 2;
  while ((i < cidBytes[1]) && (cidBytes[i] != TIMEHEADER) ) {
    /* skip to the next header */
    i = i + cidBytes[i+1] + 2;
  }
  if (i >= cidBytes[1]) return;

  Time10ms = 0;
  TimeSec = 0;
  
  i += 2;
  TimeDay = ((cidBytes[i] & 0x0f) * 10) + (cidBytes[i+1] & 0x0f);
  i += 2;
  TimeMonth = ((cidBytes[i] & 0x0f) * 10) + (cidBytes[i+1] & 0x0f);  
  i += 2;
  TimeHour = ((cidBytes[i] & 0x0f) * 10) + (cidBytes[i+1] & 0x0f);  
  i += 2;
  TimeMin = ((cidBytes[i] & 0x0f) * 10) + (cidBytes[i+1] & 0x0f);  
}

/* Update the Email string based on CID information */
/* the CID frame must be valid */ 

#define LOCNUMBER 34
#define LOCNAME 54
   
void CIDgetData(u08 *string)
{
  u08 i, j;
  u08 length;

  i = 2;
  while ((i < cidBytes[1]) && (cidBytes[i] != NUMBERHEADER) ) {
    /* skip to the next header */
    i = i + cidBytes[i+1] + 2;
  }
  if (i >= cidBytes[1]) return;
  
  i += 2;
  string[LOCNUMBER+0] = cidBytes[i+0];  
  string[LOCNUMBER+1] = cidBytes[i+1];  
  string[LOCNUMBER+2] = cidBytes[i+2];  
  string[LOCNUMBER+4] = cidBytes[i+3];  
  string[LOCNUMBER+5] = cidBytes[i+4];  
  string[LOCNUMBER+6] = cidBytes[i+5];  
  string[LOCNUMBER+8] = cidBytes[i+6];  
  string[LOCNUMBER+9] = cidBytes[i+7];  
  string[LOCNUMBER+10] = cidBytes[i+8];  
  string[LOCNUMBER+11] = cidBytes[i+9];
  
  i = 2;
  while ((i < cidBytes[1]) && (cidBytes[i] != NAMEHEADER) ) {
    /* skip to the next header */
    i = i + cidBytes[i+1] + 2;
  }
  if (i >= cidBytes[1]) return;
  
  length = cidBytes[i+1];
  if (length > 20) length =20;
  i += 2;
  for (j=0; j<length; j++) {
    string[LOCNAME+j] = cidBytes[i+j];
  }        
}      
