
#include <LPC213x.H>                       /* LPC213x definitions  */
#include "typedefs.h"
#include "spiLPC.h"

/******************************************************************************
 * Function:        void init_SPI()
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 *
 * Overview:        Set the SPI I/F
 * Note:            None
 *****************************************************************************/
void init_SPI()
{
  // setup SPI interface :
  // Master mode, MSB first
  IOSET0 = 0x00000080;
//  S0SPCR = 0x00000038;
  S0SPCR = 0x00000020;
  S0SPSR = 0x00000000;

  // clock = f/8
  S0SPCCR = 0x00000008;
}

/******************************************************************************
 * Function:        u08 spiPut (u08 valueIn)
 * PreCondition:    The spi device ,must be selected with chip select
 * Input:           value to write
 * Output:          Value read from the SPI bus
 * Side Effects:    None
 *
 * Overview:        Write a byte to the SPI H/W and wait for the end of the transfer
 *                  Return the value read 
 * Note:            None
 *****************************************************************************/
u08 spiPut (u08 valueIn)
{
  S0SPDR = valueIn;

	// wait for transfer to complete
  while ( (S0SPSR & 0x80) == 0);
  return S0SPDR;
}

