#include <stdio.h>
#include <string.h>
#include <LPC213x.H>                     /* LPC21xx definitions               */
#include "typedefs.h"
#include "timer.h"
#include "FATLib.h"
#include "dtmf.h"
#include "config.h"
#include "smtp.h"

#define DEBUG


/* General timers */
extern volatile u16 Timer10ms_1;
extern u08 Time10ms;
extern u08 TimeSec;
extern u08 TimeMin;
extern u08 TimeHour;
extern u08 TimeDay;
extern u08 TimeMonth;

extern volatile u08 dspmode;
extern u08 ledStatus, switchStatus, switchStatusLong;

/* Initial values */
char RingToAnswer = 3;
char MaxUser = '3';
char DTMFTimeOut = 40;
char RecordTimeOut = 180;
char GreetingFileName[13] = "hello.wav";
char NoUserFileName[13] = "message.wav";
char User1FileName[13] = "jim.wav";
char User2FileName[13] = "julie.wav";
char User3FileName[13] = "kids.wav";
char User1Notify[50] = "";
char User2Notify[50] = "";
char User3Notify[50] = "";
char AllNotify = '1';

#define MAXVAR 13

/* Number of 205-sample buffers needed for dial tone detection */
/* 25 ms * 40 = 1 sec */
#define DIALTONEDETECT 40

#define CURRENTGONE 40
#define CIDTIMEOUT 250

#define ERASETIMEOUT 300

char htmlFooter[] = "</body></html>\r\n";
#define FOOTERSIZE 17
char eMailNotification[] = "Received at xx:xx on xx/xx\r\nFrom: xxx-xxx-xxxx\r\nName: N/A                 \r\n";
char eMailString[100];
char uFileString[100];
char wFileString[13];
char tFileString[13];
char hFileString[13];

#define LOCTIME 12
#define LOCDATE 21
#define LOCCRLF1 26
#define LOCCRLF2 46
#define LOCCRLF3 74

u08 UserToLed[] = {0x00, 0x01, 0x03, 0x07, 0x0f};

char *initVar[] = { "RingToAnswer", "MaxUser", "DTMFTimeOut" , "RecordTimeOut",
                    "GreetingFileName", "NoUserFileName", "User1FileName", "User2FileName",
                    "User3FileName", "User1Notify", "User2Notify", "User3Notify",
                    "AllNotify" };
char initType[]= { 1, 0, 1, 1,
                   0, 0, 0, 0,
                   0, 0, 0, 0, 
                   0  };
char *initValue[] = { &RingToAnswer, &MaxUser, &DTMFTimeOut, &RecordTimeOut,
                      GreetingFileName, NoUserFileName, User1FileName, User2FileName,
                      User3FileName, User1Notify, User2Notify, User3Notify,
                      &AllNotify };

u08 *notifiedUser;

s08 playWaveFile(u08 *filename);
s08 recordWaveInit(u08 *filename);
void recordWave( s08 hWave );
void recordWaveEnd( s08 hWave );
s08 checkCIDvalid( void);
void updateCIDtime();
void CIDgetData(u08 *string);

static u08 skipComments(s08 handle);
static void wait10ms( u16 delay );
static void assignVariable(u08 *variable, u08 *value);
static void CreateEmailString( void );
static u08 ToASCIIhigh(u08 value);
static u08 ToASCIIlow(u08 value);
void createHTMLmessagesFile(void);
void localPlayback(void);

extern u08 RingCount;
extern u08 cidBytes[];
extern u08 LineCurrentGone;

/******************************************************************************
 * Function:        s08 tamtam( void )
 * PreCondition:    None
 * Input:           None
 * Output:          0 No error
 *                  -1 an error occured
 * Side Effects:    Many...
 *
 * Overview:        We come here when we have detected a ring 
 * Note:            None
 *****************************************************************************/
s08 tamtam( void )
{
  static u08 callerIDstate = NOTDONE;
  u08 user = '0';
  u08 userEmailed;
  u08 i;
  s08 hWave;
  u16 dialtonecounter;
  s08 dialdetect;

  if (RingCount == 0) {
    callerIDstate = NOTDONE;
    return 0;
  }
  
  /******************************************************************************
  * CALLER ID
  ******************************************************************************/
  if ( (RingCount == 1) && (callerIDstate == NOTDONE) ) {
    callerIDstate = INPROGRESS;
    #ifdef DEBUG
    printf("Caller ID detection \n");
    #endif
    dspmode = FSKINIT;
    Timer10ms_1 = CIDTIMEOUT;
    while (( Timer10ms_1 != 0 ) && (dspmode != IDLE) )
    {
      /* Wait ! */
    }
    if (checkCIDvalid( )) {
      callerIDstate = VALID;
      updateCIDtime();
      #ifdef DEBUG
      printf("\nDate: %d/%d\n",TimeMonth,TimeDay);
      printf("Time: %d:%d\n",TimeHour,TimeMin);
      #endif
    }  
  #ifdef DEBUG
  printf("Caller ID detection end\n");
  #endif
  dspmode = FSKEND;
  }  
  
  if (RingCount < RingToAnswer) {
    return 0;
  }

  /******************************************************************************
  * ANSWER THE CALL
  ******************************************************************************/

  /* Make sure that we only answer once */
  RingCount = 0;
  /* OFF HOOK */
  IOCLR1  = OFFHOOK;
  /* Wait a little bit for everything to settle */
  wait10ms(20);
  /* Start the timer that monitor the line current */
  LineCurrentGone = CURRENTGONE;

  /******************************************************************************
  * PLAY GREETING #1
  ******************************************************************************/
  if (playWaveFile( GreetingFileName) == -1) {
    return -1;
  }
  
  /* If the remote already hangup or the local phone is picked up,
  exit right away */
  if (LineCurrentGone == 0) return -1;

  /* Play a short tone */
  dspmode = SENDTONE;   
  wait10ms(50);
  dspmode = IDLE;   

  /******************************************************************************
  * DTMF DETECTION
  ******************************************************************************/
  DTMFactivate();
  Timer10ms_1 = DTMFTimeOut*10;
  while (( Timer10ms_1 != 0 ) && ( (user = DTMFdecode()) == NODTMF ))
  {
    /* If the remote already hangup or the local phone is picked up,
    exit right away */
    if (LineCurrentGone == 0) return -1;
  } 
  /* If no DTMF or DTMF is invalid, assume user 0 */
  if ( (user == NODTMF) || (user > MaxUser) )
  {
    user = '0';
  }
  
  /******************************************************************************
  * PLAY GREETING #2
  ******************************************************************************/
  switch (user)
  {
    case '0':
      if (playWaveFile( NoUserFileName) == -1) return -1;
      break;

    case '1':
      if (playWaveFile( User1FileName) == -1) return -1;
      break;

    case '2':
      if (playWaveFile( User2FileName) == -1) return -1;
      break;

    case '3':
      if (playWaveFile( User3FileName) == -1) return -1;
     break;
  }

  /* If the remote already hangup or the local phone is picked up,
  exit right away */
  if (LineCurrentGone == 0) return -1;
 
  /* Send an invite tone */
  dspmode = SENDTONE;   
  wait10ms(100);
  dspmode = IDLE;   
  
  /* If the remote already hangup or the local phone is picked up,
  exit right away */
  if (LineCurrentGone == 0) return -1;

  /* Get ready to detect the dial tone */
  TONEactivate();
  /* Get ready to record the remote voice */
  hWave = fat_openRead("temp.wav");
  if (hWave >= 0)
  {
    fat_close(hWave);
    hWave = fat_delete("temp.wav");
  }
 	hWave = recordWaveInit("temp.wav");
  
  /******************************************************************************
  * RECORDING
  ******************************************************************************/

  Timer10ms_1 = RecordTimeOut * 100;
  dialtonecounter = 0;
  while ( (dialtonecounter < DIALTONEDETECT) && (LineCurrentGone != 0) && (Timer10ms_1 != 0) )
  {
    recordWave(hWave);
    dialdetect = TONEdecode();
    if (dialdetect != -1)
    {
      if (dialdetect == 2) {
        dialtonecounter++;
      } else {
        dialtonecounter = 0;
      }
    }
  }
  dspmode = IDLE;             /* Stop the recording */
  recordWaveEnd(hWave);       /* Close the file */
  printf("End of recording\n");
  /* Go on HOOK */
  IOSET1  = OFFHOOK;

  /******************************************************************************
  * FIND THE FILE NAME TO USE
  ******************************************************************************/
  wFileString[0] = 'U';
  wFileString[1] = user;
  wFileString[2] = 'M';
  wFileString[5] = '.';
  wFileString[6] = 'W';
  wFileString[7] = 'A';
  wFileString[8] = 'V';

  i = 0;
  do {
    wFileString[3] = ToASCIIhigh(i);
    wFileString[4] = ToASCIIlow(i);
    if  ( (hWave=fat_openRead(wFileString) ) >= 0)
    {
      fat_close(hWave);
    } else {
      break;
    }
	  i++;
  } while (i< 100);
  if (i< 100) {
    fat_renameFile("temp.wav", wFileString);
    for (i=0; i<6; i++) tFileString[i] = wFileString[i];
    tFileString[6] = 'T';
    tFileString[7] = 'X';
    tFileString[8] = 'T';
    hWave = fat_openWrite(tFileString);
  }

  /******************************************************************************
  * EMAIL NOTIFICATION
  ******************************************************************************/

  /* Create the Email string */
  CreateEmailString();
  if (callerIDstate == VALID ) {
    CIDgetData(eMailString);
  }
  
  /* Check who should be notified */
  userEmailed = user;
  if (user == '0') {
    if ( AllNotify == '0' ) {
      return 0;
    }
    userEmailed = AllNotify;
  }

  if (userEmailed == '1') notifiedUser = User1Notify; 
  if (userEmailed == '2') notifiedUser = User2Notify; 
  if (userEmailed == '3') notifiedUser = User3Notify;
  
  if (notifiedUser[0] == 0) return 0; 

  smtp_send(notifiedUser, "<tamtam@mail.com>", "New Message", eMailString, 76);
  #ifdef DEBUG
  printf("Message sent\n");
  #endif

  /******************************************************************************
  * SAVE CID INFORMATION IN THE FILE
  ******************************************************************************/

  if (hWave < 0) return 0;
  /*remove CRLF */
  for (i=0; i<100; i++) uFileString[i] = eMailString[i];
  uFileString[LOCCRLF1+0] = ' ';
  uFileString[LOCCRLF1+1] = ' ';
  uFileString[LOCCRLF2+0] = ' ';
  uFileString[LOCCRLF2+1] = ' ';
  uFileString[LOCCRLF3+0] = ' ';
  uFileString[LOCCRLF3+1] = ' ';
  fat_write(hWave,uFileString, LOCCRLF3);
  fat_close(hWave);

  /******************************************************************************
  * BUILD A NEW HTML FILE
  ******************************************************************************/
  createHTMLmessagesFile();
  #ifdef DEBUG
  printf("HTML file created\n");
  #endif

  return 0;
}

/******************************************************************************
 * Function:        s08 readConfig( u08 *filename )
 * PreCondition:    None
 * Input:           *filename
 * Output:           0 No error
 *                  -1 an error occured
 * Side Effects:    All global variables are updated
 *
 * Overview:        Read the config file and update all the globals 
 * Note:            None
 *****************************************************************************/

s08 readConfig(u08 *filename)
{
  s08 hConfig;
  u08 variable[20];
  u08 sValue[50];
  u08 i;
  s16 size;
  u08 character;

	hConfig = fat_openRead(filename);
  if (hConfig < 0)
  {
    return -1;                                      /* Failed */
  }

  while (1)
  {
	  size = fat_read(hConfig, &character,1);                /* Read one character */
    if (size == 0)
    {
      break;
    }
    if (character == ';')
    {
      /* Skip the line of comments */
      if (skipComments(hConfig) == 0)
      {
        break;
      }
    } else {
      /* We got the name of a variable */
      i = 0;
      while (character != ':')
      {
        variable[i] = character;
        i++;
        size= fat_read(hConfig, &character, 1);     /* Read character */
      }
      variable[i] = 0;                                /* Terminate the string */

      /* remove the spaces */
      size= fat_read(hConfig, &character, 1);     /* Read character */
      while (character == ' ')
      {
        size= fat_read(hConfig, &character, 1);     /* Read character */
      }

      i = 0;
      while (character >= ' ') {
        sValue[i] = character;
        i++;
        size= fat_read(hConfig, &character, 1);     /* Read character */
      }
      sValue[i] = 0;                                /* Terminate the string */

      /* Find the end of line */
      while (character != '\n') {
        size= fat_read(hConfig, &character, 1);     /* Read character */
      }
      assignVariable(&variable[0], &sValue[0]);
    }
  }
  fat_close(hConfig);
  return 0;
}

u08 skipComments(s08 handle)
{
  u08 ch;
  s16 size;
  
  do
  { 
    size = fat_read(handle, &ch, 1);                 /* Read one character */
  } while (( ch != '\n') && (size == 1) );
  
  if (size == 0)
  {
    return 0;
  }

  return 1;
}

u16 stringComp(const u08 *string1, const u08 *string2, u16 length)
{
	int i;
	for (i=0; i<length; i++)
	{
		if (string1[i] != string2[i])
		{
			return -1;
		}
	}
	return 0;
}

u16 stringCompMax(const u08 *string1, const u08 *string2)
{
	int i;
  
  i = 0;
  while (string2[i] != 0) {
    if (string1[i] != string2[i]) return -1;
    i++;
  }
  
  if (string1[i] != 0) return -1;
	return 0;
}


void assignVariable(u08 *variable, u08 *value)
{
  u08 i, j;
  u08 *var;
  u08 result;

  i=0;
  while (i< MAXVAR) {
    if ( !stringCompMax(variable, initVar[i]) ) {
      var = initValue[i];
      if ( initType[i] == 1 ) {
        result = 0;
        j = 0;
        while (value[j] != 0) {
          result = (result*10) + (value[j] & 0xf);
          j++;
        }
        *var = result;
      } else {
        j=0;
        while (value[j] != 0) {
          var[j] = value [j];
          j++;
        }
        var[j] = value [j];
      }
      return;
    }
    i++;
  }
}
/******************************************************************************
 * Function:        void wait10ms( u16 delay )
 * PreCondition:    None
 * Input:           delay
 * Output:          None
 * Side Effects:    None
 *
 * Overview:        Wait delay x 10 ms 
 * Note:            None
 *****************************************************************************/
void wait10ms( u16 delay )
{  
    Timer10ms_1 = delay;
    while ( Timer10ms_1 != 0 )
    {
    }
}

u08 ToASCIIhigh(u08 value)
{
  value = value / 10;
  return value + 0x30;
}

u08 ToASCIIlow(u08 value)
{
  value = value % 10;
  return value + 0x30;
}


/* Create EMail string */

void CreateEmailString( void )
{
  u08 i;

  i=0;
  do {
    eMailString[i] = eMailNotification[i];
  } while (eMailNotification[i++] != 0);

  eMailString[LOCTIME+0] = ToASCIIhigh(TimeHour);
  eMailString[LOCTIME+1] = ToASCIIlow(TimeHour);
  
  eMailString[LOCTIME+3] = ToASCIIhigh(TimeMin);
  eMailString[LOCTIME+4] = ToASCIIlow(TimeMin);

  eMailString[LOCDATE+0] = ToASCIIhigh(TimeMonth);
  eMailString[LOCDATE+1] = ToASCIIlow(TimeMonth);
  
  eMailString[LOCDATE+3] = ToASCIIhigh(TimeDay);
  eMailString[LOCDATE+4] = ToASCIIlow(TimeDay);
}

void createHTMLmessagesFile()
{
  s08 hHfile;
  s08 hTfile;
  s08 hWfile;
  u08 wFileString[13];
  u08 tFileString[13];
  u08 i,j;
  u08 currentUser;
  u08 led;

  u08 length;
  u08 buffer[100];
              
  /* Default is no blinking LED */
  ledStatus =0x00;
                                   
  hHfile = fat_openRead("messages.htm");
  if (hHfile >= 0)
  {
    fat_close(hHfile);
    hHfile = fat_delete("messages.htm");
  }
 	hHfile = fat_openWrite("messages.htm");
  fat_write(hHfile, "<html><body>\r\n", 14);
  led = 0x10;
  for (currentUser = '0'; currentUser<'4'; currentUser++) {
    led = led >> 1;
    fat_write(hHfile, "<h2>Messages for User", 21);
    fat_write(hHfile, &currentUser, 1);
    fat_write(hHfile, "</h2>\r\n",7);

    wFileString[0] = 'U';
    wFileString[1] = currentUser;
    wFileString[2] = 'M';
    wFileString[5] = '.';
    wFileString[6] = 'W';
    wFileString[7] = 'A';
    wFileString[8] = 'V';
    wFileString[9] = 0;

    i = 0;
    do {
      wFileString[3] = ToASCIIhigh(i);
      wFileString[4] = ToASCIIlow(i);
      if  ( (hWfile=fat_openRead(wFileString) ) >= 0)
      {
        fat_close(hWfile);
        /* WE HAVE A MESSAGE PENDING FOR THAT USER */
        ledStatus = ledStatus | led;
        for (j=0; j<10; j++) tFileString[j] = wFileString[j];
        tFileString[6] = 'T';
        tFileString[7] = 'X';
        tFileString[8] = 'T';
        hTfile = fat_openRead(tFileString);
        fat_write(hHfile,"<p><a href=\"", 12);
        fat_write(hHfile, wFileString, 9 );						
        fat_write(hHfile, "\">", 2 );						
        length = fat_read(hTfile, &buffer[0], 100 );	
        fat_close(hTfile);					
        fat_write(hHfile, &buffer[0], length );						
        fat_write(hHfile, "</a></p>\r\n", 10 );						
      } else {
          break;
      }
	      i++;
    } while (i< 100);
    if (i==0) {
      fat_write(hHfile,"<p>None</p>", 11);
    }     
  }
  fat_write(hHfile, htmlFooter, FOOTERSIZE);
  fat_close(hHfile);
}
void localPlayback( void )
{
  u08 user;
  u08 wFileString[13];
  u08 tFileString[13];
  u08 i, j;
  s08 hWfile;
  u08 rebuild = 0;

  if (switchStatus == 1) {
    user = '0';
    } else if (switchStatus == 2) {
        user = '1';
      } else if (switchStatus == 4) {
          user = '2';
        } else if (switchStatus == 8) {
            user = '3';
        } else {
          return;
        }

  switchStatus = 0;

  wFileString[0] = 'U';
  wFileString[1] = user;
  wFileString[2] = 'M';
  wFileString[5] = '.';
  wFileString[6] = 'W';
  wFileString[7] = 'A';
  wFileString[8] = 'V';
  wFileString[9] = 0;
  i = 0;
  do {
    wFileString[3] = ToASCIIhigh(i);
    wFileString[4] = ToASCIIlow(i);
    if  ( (hWfile=fat_openRead(wFileString) ) >= 0)
    {
      /* WE HAVE A MESSAGE PENDING FOR THAT USER */
      fat_close(hWfile);
      playWaveFile(wFileString);
      for (j=0; j<10; j++) tFileString[j] = wFileString[j];
      tFileString[6] = 'T';
      tFileString[7] = 'X';
      tFileString[8] = 'T';

      Timer10ms_1 = ERASETIMEOUT;
      /* Play a very short tone */
      dspmode = SENDTONE;   
      wait10ms(20);
      dspmode = IDLE;
      while ( Timer10ms_1 != 0 ) {
        if (switchStatusLong != 0 ) {
          rebuild = 1;
          fat_delete(wFileString);
          fat_delete(tFileString);
          /* Play a short tone */
          dspmode = SENDTONE;   
          wait10ms(50);
          dspmode = IDLE;
          while ((IOPIN1 & (SW0|SW1|SW2|SW3)) != (SW0|SW1|SW2|SW3));
          switchStatusLong = 0;
          switchStatus = 0;
          break;
        }
      }
    } else {
       break;
    }
    i++;
  } while (i< 100);

  /******************************************************************************
  * BUILD A NEW HTML FILE
  ******************************************************************************/
  if (rebuild) {
    createHTMLmessagesFile();
    #ifdef DEBUG
    printf("HTML file created\n");
    #endif
  }
}
