/******************************************************************************/
/*                                                                            */
/*  timer.c:  Low Level Timer Routines          	                            */
/*                                                                            */
/******************************************************************************/

#include <LPC213x.H>                     /* LPC21xx definitions               */
#include <math.h>
#include <stdio.h>
#include "config.h"
#include "typedefs.h"
#include "timer.h"
#include "dtmf.h"
#include "cid.h"

u08 ledStatus;                  /* LED status: on or blinking */
u08 switchStatus;               /* Switch status: on or off */
u08 switchStatusLong;           /* Switch status: on or off */
u08 dspmode;										/* Dispatch the Timer IRQ */

s16 *pWaveSample;          		/* Current pointer to the wave sample  - Ping*/
s16 *pWaveEnd;					  		/* End pointer to the wave sample */
s16 *pWaveSampleNext;					/* Next pointer  - Pong */
s16 *pWaveEndNext;			  		/* Next end */

extern s32 DTMFsample1[DTMF_NPOINTS];	/* DTMF samples - Ping */
extern s32 DTMFsample2[DTMF_NPOINTS];	/* DTMF samples - Pong */

s32 *p_DTMFsample;          		/* Current pointer to the DTMF sample  - Ping*/
s32 *p_DTMFend;					  			/* End pointer to the wave sample */

u08 DTMFbufferReady;						/* Indicates if buffer 1 or 2 is ready for processing */
u08 waveReady;
u08 LineCurrentGone;

/* General purpose timers */
u16 TimerIP10ms;
u16 Timer10ms_1;
u16 MaxRing10ms;
u16 RingCount10ms =0 ;
u16 InterRing =0 ;

u08 Time10ms = 0;
u08 TimeSec = 0;
u08 TimeMin = 0;
u08 TimeHour = 0;
u08 TimeDay = 1;
u08 TimeMonth = 1;

static u08 DayPerMonth[12] ={31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30 ,31};

/* Ring Counter */
static u08 RingTransition = 0;
u08 RingCount = 0;


static s32 coeffloi[NPERBAUD];				/* Coefficient: Cos 1200 Hz */
static s32 coeffloq[NPERBAUD];				/* Coefficient: Sin 1200 Hz */
static s32 coeffhii[NPERBAUD];				/* Coefficient: Cos 2200 Hz */
static s32 coeffhiq[NPERBAUD];				/* Coefficient: Sin 2200 Hz */

static u16 tone[80];            				/* 1000 Hz tone */

extern s16 wavedata1[WAVESAMPLES];
extern s16 wavedata2[WAVESAMPLES];

#define IRQ8000	3750					/* (EPB / 8000)	*/
#define IRQ9600	3125					/* (EPB / 9600)	*/
#define IRQ10ms 3000          /* (EPB / 100 / 100 */
#define AFTERRINGTIMER 10     /* Detect the end of a ring Unit of 10 ms */ 
#define INTERRINGTIMER 1000        /* If no Ring, reset the ring count Unit of 10 ms */ 
/* The ring detection give us full wave detection so
   the frequency is actually doubled */
#define MINRINGFREQUENCY 15
#define MAXRINGFREQUENCY 25

/* Timer Counter 0 Interrupt */
void tc0_IRQ (void)
{
	static u08 rxbyte;	
	u32 ADCvalue;
	s16 DACvalue;

/* for FSK demodulation */
	s32 outloi=0, outloq=0, outhii=0, outhiq=0;
  s32 out, sample;
	u08	i;
	static u08 rxbit, previousbit;
	static u16 phase;
  static u08 FSKstatus;
  static s16 rxsamples[NPERBAUD];		  	/* input queue */
  static u08 rxptr = 0;			            /* Pointer in the RX queue */
  static u08 toneIndex = 0;

	T0IR        = 1;             	/* Clear interrupt flag */
	VICVectAddr = 0;             	/* Acknowledge Interrupt */

  switch (dspmode) {
   
    /*State: Idleing */
		case IDLE:
      break;

    /* State: Playing */
    case PLAYWAVE:
      DACvalue = *pWaveSample++;												/* Get next wave sample */
      DACR = (DACvalue + 0x8000);												/* Convert from signed to unsigned */
      if (pWaveSample >= pWaveEnd)										  /* End of buffer? */
      {
        if (pWaveSampleNext == 0)											  /* Check if another buffer is ready */
        {
          dspmode = IDLE;																/* No, stop playing */
        } else {
        	pWaveSample = pWaveSampleNext;							  /* Yes, update head and tail */
          pWaveEnd = pWaveEndNext;
          pWaveSampleNext = 0;												  /* Notify background that the buffer was consumed */
        }
      }
      break;

    /* State: Recording */
    case RECORDWAVE:
		 	ADCvalue = AD0DR;       									        /* Get the sample */
 			AD0CR |= 0x01200000;                        			/* Start A/D Conversion for next sample	*/
			*pWaveSample++ = ADCvalue - - 0x8000;							/* Convert to signed, store and increment pointer */
			if (pWaveSample >= pWaveEnd)										  /* Did we reach the end of the current buffer ? */
			{
				if (pWaveEnd == (&wavedata1[0] + WAVESAMPLES))	/* Check which buffer we just finished */
				{
					pWaveSample = &wavedata2[0];									/* Init pointer */
					pWaveEnd = &wavedata2[0] + WAVESAMPLES;				/* Init end */
					waveReady = 1;																/* Store buffer # 1 */
				} else {
					pWaveSample = &wavedata1[0];									/* Init pointer */
					pWaveEnd = &wavedata1[0] + WAVESAMPLES;				/* Init end */
					waveReady = 2;																/* Store buffer # 2 */
				}
			}
      ADCvalue = ((ADCvalue >> 7) & 0x1FF) - 0x100;    	/* Scale the A to D value to signed 8 bits = 9 bits */
      goto TONEENTRY;                                         	
//			break;

		/* State: Detecting DTMF */
		case DTMFDETECT:
      ADCvalue = ((AD0DR >> 7) & 0x1FF) - 0x100;       	/* Scale the A to D value to signed 8 bits = 9 bits */
 			AD0CR |= 0x01200000;                        			/* Start A/D Conversion for next sample	*/
	  TONEENTRY:
  		*p_DTMFsample++ = ADCvalue;												/* Save the current sample */
			if (p_DTMFsample >= p_DTMFend)										/* Did we reach the end of the current buffer ? */
			{
				if (p_DTMFend == (&DTMFsample1[0] + DTMF_NPOINTS)) /* Check which buffer we just finished */
				{
					p_DTMFsample = &DTMFsample2[0];								/* Init pointer */
					p_DTMFend = &DTMFsample2[0] + DTMF_NPOINTS;		/* Init end */
					DTMFbufferReady = 1;													/* Process buffer # 1 */
				} else {
					p_DTMFsample = &DTMFsample1[0];								/* Init pointer */
					p_DTMFend = &DTMFsample1[0] + DTMF_NPOINTS;		/* Init end */
					DTMFbufferReady = 2;													/* Process buffer # 2 */
				}
			}	
			break;

		case FSKINIT:
			/* Change the interruupt rate to 9600 Hz*/
  		T0MR0 = IRQ9600;                              		/* Sampling rate = 9600 Hz */
      CIDbyte(0, TRUE);
			FSKstatus = 10;																		/* Wait for a mark */
			rxptr = 0;
			dspmode = FSK;																		/* Go to FSK mode */
      break;

		case FSKEND:
			/* Change the interrupt rate back to 8000 Hz*/
  		T0MR0 = IRQ8000;                              		/* Sampling rate = 8000 Hz */
			dspmode = IDLE;																		/* Go to IDLE mode */
			break;

		case FSK:
		 	ADCvalue = ((AD0DR >> 1) & 0x7fff) - 0x4000;      /* Convert from unsigned to signed */
 			AD0CR |= 0x01200000;                        			/* Start A/D Conversion for next sample	*/
    	rxsamples[rxptr] = ADCvalue;
    	rxptr = (rxptr + 1) % NPERBAUD;

	    for (i=0; i<NPERBAUD; i++)
      {
		    sample = rxsamples[(rxptr+i) % NPERBAUD];
		    outloi += ( sample * coeffloi[i]);
   		  outloq += ( sample * coeffloq[i]);
		    outhii += ( sample * coeffhii[i]);
		    outhiq += ( sample * coeffhiq[i]);
	    }

	    out =   (outhii>>15) * (outhii>>15) + (outhiq>>15) * (outhiq>>15)
		        - (outloi>>15) * (outloi>>15) - (outloq>>15) * (outloq>>15);

	    if (out >= 0)
      {
		    rxbit = 0x00;                               /* Bit is zero */
		  } else {                                      /* Bit is one */
		    rxbit = 0x80;     
		  }
#ifdef SYNCHRONOUS
      /* adjust the phase of the sampling point */
	    if (rxbit != previousbit)
      {
		    if (phase < PHASEHALF)
        {
			    phase += PHASEADJUST;
			  } else {
			    phase -= PHASEADJUST;
			  }
 		  }
#endif 

			if ( (previousbit != 0) && (rxbit == 0) && (FSKstatus == 10)	)
			{
				FSKstatus = 9;
				phase = PHASEHALF;
			}

      /* Is it time to sample the bit ? */
	    if (phase >= PHASEMAX)
      {
        switch (FSKstatus)
        {
           /* Receive a mark while in hunt mode */
           case 10:
           break;
           
           case 9:
           if (rxbit == 0)
				    {
					    /* We detected a start bit, get ready for the 8 bits of data */
					    FSKstatus = 8;
				    } else {
							/* We had a false start, go back hunting */
							FSKstatus = 10;
						}
            break;
            
          case 0:
					  /* Check if the bit is really a stop */
					  if (rxbit != 0)
					  {
						  /* We got a stop bit, store the character */
              CIDbyte(rxbyte, FALSE);
					  } else {
              /* Bad character reset the state machine */
							CIDbyte(0, TRUE);
            }
   					FSKstatus = 10;																/* Wait for a start again */
            break;
            
          default:
            /* we are getting the data bits */
					  rxbyte = rxbyte >> 1;                         /* Shift and */
					  rxbyte += rxbit;        											/* Add current bit */
					  FSKstatus--;
        }
      }
 	    previousbit = rxbit;
      phase = phase % PHASEMAX;
			phase += PHASEINC;
			break;

    case SENDTONE:
      DACR = tone[toneIndex];     										/* Send the next sample*/
      toneIndex = (toneIndex + 1) % 8;                /* Next index */
      break;

  } /* switch */
}

void initFSK(void)
{
  u08 i;

  for (i=0; i<NPERBAUD; i++)
  {
    coeffloi[i] = 16383*cos(2*PI*i/NPERBAUD*1200/1200);
  	coeffloq[i] = 16383*sin(2*PI*i/NPERBAUD*1200/1200);
		coeffhii[i] = 16383*cos(2*PI*i/NPERBAUD*2200/1200);
	  coeffhiq[i] = 16383*sin(2*PI*i/NPERBAUD*2200/1200);
	}
}

/* Timer Counter 1 Interrupt */
void tc1_IRQ (void)
{
	static u08 Timer10ms_LED = 0;
	static u08 Timer10ms_SW = 0;
  u16 frequency;
  u08 switchTemp1;
  static u08 switchTemp2;

  T1IR        = 1;             	/* Clear interrupt flag */
	VICVectAddr = 0;             	/* Acknowledge Interrupt */
  
  if (TimerIP10ms != 0 )   TimerIP10ms--;
  if (Timer10ms_1 != 0 )   Timer10ms_1--;
  if (InterRing != 0 )     InterRing--;
  if (InterRing == 0)       RingCount = 0;

  /* Ring processing */
  RingCount10ms++;
  if (MaxRing10ms != 0 )
  {
    MaxRing10ms--;
    if (MaxRing10ms == 0) {
      /* Ignore too short rings */
      if (RingTransition > 10) {
        /* Divide by two to take the ful wave rectification into account */
				frequency = (RingTransition*100)/(RingCount10ms-AFTERRINGTIMER)/2;
        /* Check the frequency */
        if ((frequency > MINRINGFREQUENCY) && (frequency < MAXRINGFREQUENCY)) {
          /* Increment the global Ring counter */
          RingCount++;
          InterRing = INTERRINGTIMER;
        }
      }
      RingTransition = 0;                                    /* Reset for next time */
     }
  }
  /*-----------------------------------------------------------------------*/
  /* Check if the line current has disappeared or if the phone current
  has appeared */
  if ( (IOPIN0 & SENSEDAA) || !(IOPIN0 & SENSEPHONE) ) 
  {	
    if (LineCurrentGone != 0) LineCurrentGone--;
  } 
  /*-----------------------------------------------------------------------*/
  /* Time Of Day */
  Time10ms++;
  if ( Time10ms == 100 ) {
    Time10ms = 0;
    TimeSec++;
    if ( TimeSec == 60) {
      TimeSec = 0;
      TimeMin++;
      if ( TimeMin == 60) {
        TimeMin = 0;
        TimeHour++;
        if ( TimeHour == 24) {
          TimeHour = 0;
          if ( TimeDay == DayPerMonth[TimeMonth] ) {
            TimeDay = 1;
            TimeMonth = (TimeMonth % 12) + 1;
          } else {
            TimeDay++;
          }
        }
      }      
    }
  }
  /*-----------------------------------------------------------------------*/
  /* LED's */
  /*-----------------------------------------------------------------------*/
  Timer10ms_LED++;
 
  if( (IOPIN0 & CARDDETECT) != 0)
  {	
	  if ( (Timer10ms_LED & 0x1F) > 0x10 ) {
      IOSET1 = LED0 + LED1 + LED2 + LED3;
    } else {
      IOCLR1 = LED0 + LED1 + LED2 + LED3;
    }
  } else {
	  if ( (Timer10ms_LED & 0x7f) > 0x40 ) {
      IOSET1 = LED0 + LED1 + LED2 + LED3;
    } else {
      IOCLR1 = (u32) (ledStatus << 16);
    }
  }
  /*-----------------------------------------------------------------------*/
  /* Switch handling */
  /*-----------------------------------------------------------------------*/
  switchTemp1  = 0;
  if ((IOPIN1 & SW0) == 0) switchTemp1 += 1;
  if ((IOPIN1 & SW1) == 0) switchTemp1 += 2;
  if ((IOPIN1 & SW2) == 0) switchTemp1 += 4;
  if ((IOPIN1 & SW3) == 0) switchTemp1 += 8;
  if (switchTemp1 == switchTemp2) {
    Timer10ms_SW++;
    if (Timer10ms_SW > 100) {
      switchStatusLong = switchTemp2;
    }
   } else {
    if ( (switchTemp1 ==0) && (switchTemp2 !=0) ) {
      /* switch just released */
      if (Timer10ms_SW > 10)  switchStatus = switchTemp2;
    }
    switchTemp2 = switchTemp1;
    Timer10ms_SW = 0;
  } 
}

/* External Interrupt 0 Interrupt */
void eint0_IRQ (void)
{
  EXTINT = 1;                     /* Clear the interrupt */
 	VICVectAddr = 0;             	  /* Acknowledge Interrupt */
   if (RingTransition == 0)
  {
    RingCount10ms = 0;                /* */
  }
  /* Increment the ring counter */
  RingTransition += 1;
  MaxRing10ms = AFTERRINGTIMER;                 /* Maximum time to wait for a ring */
}

/* Setup the Interrupts */
void init_interrupts (void) {
  u08 i;
  /* Timer 0 */
  T0MR0 = IRQ8000;                              /* Sampling rate = 8000 Hz      */
  T0MCR = 3;                                     /* Interrupt and Reset on MR0   */
  T0TCR = 1;                                    /* Timer0 Enable                */
  VICVectAddr0 = (unsigned long)tc0_IRQ;        /* Set interrupt vector in 0    */
  VICVectCntl0 = 0x20 | 4;                      /* Use it for Timer 0 Interrupt */
  dspmode = IDLE;                               /* Do nothing ! */

  /* Timer 0 */
  T1MR0 = IRQ10ms;                              /* Rate = 1000Hz                */
  T1PR = 100;                                   /* Divide by 100                */
  T1MCR = 3;                                    /* Interrupt and Reset on MR0   */
  T1TCR = 1;                                    /* Timer1 Enable                */
  VICVectAddr1 = (unsigned long)tc1_IRQ;        /* Set interrupt vector in 1    */
  VICVectCntl1 = 0x20 | 5;                      /* Use it for Timer 1 Interrupt */

  /* Eint 0 */
  VICVectAddr2 = (unsigned long)eint0_IRQ;      /* Set interrupt vector in 2    */
  VICVectCntl2 = 0x20 | 14;                     /* Use it for Eint 0            */


  /* Init the tone samples */
  for (i=0; i<8; i++)
  {
    tone[i] = 0x4000*cos(2*PI* i * 2000/8000) + 0x4000;
  }
}


