#include <stdio.h>
#include <string.h>
#include <LPC213x.H>                     /* LPC21xx definitions               */
#include "typedefs.h"
#include "timer.h"
#include "FATLib.h"

extern s16 *pWaveSample;          /* Pointer to the wave sample */
extern s16 *pWaveEnd;
extern s16 *pWaveSampleNext;
extern s16 *pWaveEndNext;

extern u08 dspmode;
extern u08 waveReady;

static u16 stringComp(const u08 *string1, const u08 *string2, u16 length);

u08 readingbuffer;

s16 wavedata1[WAVESAMPLES];
s16 wavedata2[WAVESAMPLES];

#define WAVEHEADER    46
#define SIZEOFFMT     18
#define AUDIOFORMAT   1
#define NUMCHANNELS   1
#define SAMPLERATE    8000
#define BITSPERSAMPLE 16
#define BYTERATE      (SAMPLERATE * BITSPERSAMPLE / 8)
#define BLOCKALIGN    (BITSPERSAMPLE / 8) * NUMCHANNELS


/******************************************************************************
 * Function:        s08 playWave( u08 *filename )
 * PreCondition:    Timer interrupt must be configured for 8000 Hz
 * Input:           filename
 * Output:           0 the file was played correctly
 *                  -1 an error occured
 * Side Effects:    None
 *
 * Overview:        Play a wave file 
 * Note:            None
 *****************************************************************************/

s08 playWaveFile(u08 *filename)
{
  s08 returncode;
  s08 hWave;
	u32 ChunkSize;
	u08	id[4];
	u08	Subchunk1ID[4];
	u32 Subchunk1Size, SampleRate, ByteRate;
  u16	AudioFormat, NumChannels, BlockAlign, BitsPerSample, ExtraParamSize;
	u32 Subchunk2Size;
	u32 Subchunk3Size;
	
  s32 numberofbytes;
  u32 bytecounter;	

  u08 cValue;

	returncode = 0;
	hWave = fat_openRead(filename);
#ifdef DEBUG
    printf("Handle: %d\n", hWave);
#endif
  if (hWave >= 0)
  {
		fat_read(hWave, (u08*) &id, 4);                     // Get the "RIFF" header
#ifdef DEBUG
    printf("First header: %c%c%c%c\n", id[0],id[1],id[2],id[3]);
#endif
    if (!stringComp(id, "RIFF",4))
		{ //we had 'RIFF' let's continue
			fat_read(hWave, (u08*) &ChunkSize, sizeof(u32) );			      // Get the overall size
			fat_read(hWave, (u08*) &id, sizeof(u08) * 4);		   		      // Get the "WAVE" header
#ifdef DEBUG
      printf("Second header: %c%c%c%c\n", id[0],id[1],id[2],id[3]);
#endif
			if (!stringComp(id,"WAVE",4))
			{ 
				fat_read(hWave, (u08*) &Subchunk1ID, sizeof(u08) * 4 );	// Get the "fmt " header
				fat_read(hWave, (u08*) &Subchunk1Size, sizeof(u32) );		// Get the size of the fmt block
				fat_read(hWave, (u08*) &AudioFormat, sizeof(u16) );   	// Get the audio format
				fat_read(hWave, (u08*) &NumChannels, sizeof(u16) );		  // Get the # of channels
				fat_read(hWave, (u08*) &SampleRate, sizeof(u32) );			// Get the sampling rate
				fat_read(hWave, (u08*) &ByteRate, sizeof(u32) );				// Get the byte rate
				fat_read(hWave, (u08*) &BlockAlign, sizeof(u16) );			// Get the # of bytes /sample for all channels
				fat_read(hWave, (u08*) &BitsPerSample, sizeof(u16) );		// Get the # of bits per sample (should be 16)
				if (Subchunk1Size > 16)
				{
					fat_read(hWave, (u08*)&ExtraParamSize, sizeof(u16) );	// Check if there is more to read
				}

				fat_read(hWave, (u08*) &id, sizeof(u08) *4 );							// Get the next header. It will be "fact" or "data"
        while (stringComp(id, "data",4))			    				      	// Skip the segments until we get "data"
				{
					fat_read(hWave, (u08*) &Subchunk2Size, sizeof(u32) );		// Get the size of the block
          for (bytecounter = 0; bytecounter < Subchunk2Size; bytecounter++) {
  					fat_read(hWave, &cValue, sizeof(u08));	  				    // Skip all the bytes
          }
  				fat_read(hWave, (u08*) &id, sizeof(u08) *4 );						// Get the next header. It should be "data"

        }

				fat_read(hWave, (u08*) &Subchunk3Size, sizeof(u32) );      // # of bytes of sound data
		
        bytecounter = Subchunk3Size;
#ifdef DEBUG
    printf("Total Size: %ld\n", bytecounter);
#endif
        pWaveSampleNext = 0;
        pWaveSample = &wavedata1[0];
        numberofbytes = fat_read(hWave, (u08*) wavedata1, sizeof(s16)* WAVESAMPLES);
 				pWaveEnd = &wavedata1[0] + WAVESAMPLES;
        bytecounter = bytecounter - numberofbytes;
        readingbuffer = 2;
        dspmode = PLAYWAVE;
        while (bytecounter > 0)
        {
        	if (pWaveSampleNext == 0)
          {
          	if (readingbuffer == 1)
            {
            	numberofbytes = fat_read(hWave, (u08*) wavedata1, sizeof(s16)* WAVESAMPLES);
              pWaveEndNext = &wavedata1[0] + (numberofbytes / sizeof(u16) );
              pWaveSampleNext = &wavedata1[0];
              readingbuffer = 2;
           } else {
              numberofbytes = fat_read(hWave, (u08*) wavedata2, sizeof(u16)* WAVESAMPLES);
           	  pWaveEndNext = &wavedata2[0] + (numberofbytes / sizeof(u16) );
              pWaveSampleNext = &wavedata2[0];
              readingbuffer = 1;
            }
            bytecounter = bytecounter - numberofbytes;
          }
        }
        pWaveSampleNext = 0;
  		} else {
				/* Error: RIFF file but not a wave file */
#ifdef DEBUG
      printf("Error: not a WAVE file\n");
#endif
				returncode = -1;
			}
		}	else {
			/* Error: not a RIFF file */
#ifdef DEBUG
      printf("Error: not a RIFF file\n");
#endif
			returncode = -1;
		}
  }
	fat_close(hWave);
	return returncode;
}


/******************************************************************************
 * Function:        s08 recordWaveInit( u08 *filename )
 * PreCondition:    Timer interrupt must be configured for 8000 Hz
 * Input:           filename
 * Output:           >=0 the file was opened
 *                  -1 an error occured
 * Side Effects:    None
 *
 * Overview:        Start the recording a wave file 
 * Note:            None
 *****************************************************************************/

s08 recordWaveInit(u08 *filename)
{
  s08 returncode;
  s08 hWave;
  u08 u08Value;

  u32 i;

	returncode = 0;
	hWave = fat_openWrite("temp.wav");
  #ifdef DEBUG
    printf("open Write %d \n", hWave);
  #endif
  if (hWave >= 0)
  {
    /* First reserve space for the wave headers */
    u08Value = 0;
    for (i=0; i< WAVEHEADER; i++)
    {
      fat_write(hWave, &u08Value, sizeof(u08));              // Write the dummy bytes
    }
    
    /* now we record after the fake header */
    waveReady = 0;                                          /* Buffer not ready */
 		pWaveSample = &wavedata1[0];									        	/* Init pointer */
		pWaveEnd = &wavedata1[0] + WAVESAMPLES;					        /* Init end */
    dspmode = RECORDWAVE;
  }
  return hWave;
}

/******************************************************************************
 * Function:        void recordWave( s08 hWave )
 * PreCondition:    recordWave must have been called
 * Input:           filename handle
 * Output:          None
 *                  
 * Side Effects:    None
 *
 * Overview:        Perform the flash write if a buffer is ready 
 * Note:            None
 *****************************************************************************/

void recordWave( s08 hWave )
{

  u32 i;

  i = 0;
  while (i< 4)
  {
    if (waveReady != 0)
    {
      if (waveReady == 1)
      {
        fat_write(hWave, (u08*) wavedata1, sizeof(s16)* WAVESAMPLES);  // Write the samples
//        bytecounter +=  WAVESAMPLES * sizeof(s16);
      } else {
        fat_write(hWave, (u08*) wavedata2, sizeof(s16)* WAVESAMPLES);  // Write the samples
//        bytecounter +=  WAVESAMPLES * sizeof(s16);
      }
      waveReady = 0;
      i++;
    }
  }
}  

/******************************************************************************
 * Function:        s08 recordWaveEnd( s08 hWave )
 * PreCondition:    recordWave must have been called
 * Input:           filename handle
 * Output:          None
 *                  
 * Side Effects:    None
 *
 * Overview:        Close the file  
 * Note:            None
 *****************************************************************************/

void recordWaveEnd ( s08 hWave )
{
  u32 u32Value;
  u16 u16Value;

  u32 bytecounter;

  dspmode = IDLE;
  /* Update the file size since it won't change */
  fat_flush();    

  /* Find the size of the file */
  bytecounter = fatFindFileSizeWrite(hWave);

  /* Go back to the start of file to write the headers */
  fat_rewindWrite(hWave);
  
  /* RIFF header */
  fat_write(hWave, "RIFF", 4);                            // Write the "RIFF" header
   
  u32Value = bytecounter - 8;             	              // Get the size (remove RIFF and that word)
	fat_write(hWave, (u08*) &u32Value, sizeof(u32) );			  // Write the overall size
	/* Wave header */
  fat_write(hWave, "WAVE", sizeof(u08) * 4);		   		    // Write the "WAVE" header
	/* fmt header */
  fat_write(hWave, "fmt ",sizeof(u08) * 4 );      		    // Write the "fmt " header
	u32Value = SIZEOFFMT;                                   // Get the size
  fat_write(hWave, (u08*) &u32Value, sizeof(u32) );    		// Write the size of the fmt block
  u16Value = AUDIOFORMAT;                                 // Get the format
	fat_write(hWave, (u08*) &u16Value, sizeof(u16) ); 			// Write the audio format
  u16Value = NUMCHANNELS;                                 // Get the # of channels
	fat_write(hWave, (u08*) &u16Value, sizeof(u16) );				// Write it
 	u32Value = SAMPLERATE;                                  // Get the sample rate
  fat_write(hWave, (u08*) &u32Value, sizeof(u32) );    		// Write it
 	u32Value = BYTERATE;                                    // Get the byte rate
  fat_write(hWave, (u08*) &u32Value, sizeof(u32) );    		// Write it
  u16Value = BLOCKALIGN;                                  // Get the block align
	fat_write(hWave, (u08*) &u16Value, sizeof(u16) );				// Write it
  u16Value = BITSPERSAMPLE;                               // Get the # of bits per sample
  fat_write(hWave, (u08*) &u16Value, sizeof(u16) );				// Write it
  u16Value = 0;                                           // Get the extra parameter
  fat_write(hWave, (u08*) &u16Value, sizeof(u16) );			  // Write it
  /* End of fmt */
    
  /* data header */
  fat_write(hWave, "data",sizeof(u08) * 4 );      		    // Write the "data" header
  u32Value = bytecounter - WAVEHEADER;                    // Get the size of the samples
  fat_write(hWave, (u08 *) &u32Value, sizeof(u32) );      // Write it

  /* Make sure that we don't update the size of the file */
  fat_no_flush(hWave);
  /* close the file w/o update */
  fat_close(hWave);
}

u16 stringComp(const u08 *string1, const u08 *string2, u16 length)
{
	int i;
	for (i=0; i<length; i++)
	{
		if (string1[i] != string2[i])
		{
			return -1;
		}
	}
	return 0;
}


